$ = require 'jquery'
debug = require 'fe-logger'
logger = debug('npm.fe-jquery-plugin')


logger.moduleInitStart()

###*
 * Abstract Base Class that provides a base
 * for jquery plugins
 * @type {JqueryPlugin}
###
class JqueryPlugin

	###*
	 * Default options
	 * class properties
	 * @type {Object} `defaults`
	###
	@defaults: {}

	###*
 	 * The default constructor calls the initialize
	 * method and set the jQuery element.
	 * Remember to call super in subclasses if you want
	 * to maintain this behaviour.
	 * @param  {JqueryObject} `@element`
	 * @param  {Object|String} `options`
	###
	constructor: (@element, options, args...) ->
		@initialize options, args...

	###*
	 * Method to initialize the plugin instance with the given options
	 * This method could be called
	 * [initialize description]
	 * @param  {Object|String} `@options`
	###
	initialize: (@options, args...) ->
		logger.log "init: #{@constructor.name}"
		@element.trigger "ready.#{@constructor.name}"

	destroy: ->
		logger.log "destroying: #{@constructor.name}"
		@element.trigger "destroyed.#{@constructor.name}"

	###*
	 * Syntatic Sugar for bind plugin methods to events
	 * @param  {JqueryDomElement} `element`
	 * @param  {DomEvent} `event`
	 * @param  {Function} `handler`
	###
	on: (element, event, handler)->
		logger.log "event [#{event.type}] for: #{@constructor.name}"
		element.on event, handler.bind(@)

	###*
	 * Register a class as a jQuery plugin.
	 * Assuming that myClass extends jQueryPlugin
	 * it can than be registered with:
	 *
	 * 		class Foo extends Plugin
	 *
	 * 		Foo.register()
	 *
	 * This step also sets up a reference to the
	 * instance of the plugin as a data attribute
	 *
	 * If calling same plugin on an element that
	 * is already associated with the plugin, calling it
	 * again with a string (instead of an object of options)
	 * will attempt to call that methodName on the plugin.
	 *
	 * 		#creates the instance
	 * 		$(document).fooPlugin({})
	 *   	...
	 *    	# runs FooPluginInstance.open()
	 *    	$(document).fooPlugin('open')
	 *
	 *
	 * @param  {String} `pluginName` defaults to @name
	 * @returns {JqueryPlugin}
	###
	@register: (pluginName = @name) ->
		logger.log "registering: #{pluginName}"

		pluginClass = @
		methodName = null
		optionType = null

		$.fn[pluginName] = (options, args...) ->

			InvokedOptionType = $.type options

			if InvokedOptionType is 'string'
				methodName = options

			else
				options = $.extend pluginClass.defaults, options or {}


			return @each () ->
				$element = $(@)
				instance = $element.data pluginName
				options = $.extend options, $element.data() or {}

				if instance?
					# jquery element already has plugin on it.

					# run the function of the plugin
					#  $(document).fooPlugin('open')
					#  => FooPlugin.open()
					if methodName
						logger.log "running plugin method: #{pluginName}.#{methodName}"

						return instance[methodName].apply instance, args

					# else lets set the options
					else if $.type(options) is 'object' and instance.initialize?
						logger.log "setting plugin instance options: #{pluginName}"
						instance.initialize.apply instance, options

				# create a new instance
				else
					plugin = new pluginClass $element, options, args...
					plugin.name = pluginName
					logger.log "creating plugin instance: #{pluginName}"
					# store a reference to the plugin instance

					# $("#foo").data('fooPlugin')('open')
					$element.data pluginName, plugin

					# mark it as being affected by this plugin.
					$element.addClass pluginName

					# bind a clean up event.
					$element.bind "destroyed.#{pluginName}", () ->
						$element.removeData pluginName
						$element.removeClass pluginName
						$element.unbind pluginName
						plugin.destructor()

					plugin


module.exports = JqueryPlugin
